/***************************************************************************
                          kmylistbox.cpp  -  description
                             -------------------
    begin                : Tue Oct 16 2001
    copyright            : (C) 2001 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// QT includes
#include <qdir.h>
#include <qdragobject.h>
#include <qpainter.h>
#include <qregexp.h>

// KDE includes
#include <kapplication.h>
#include <kdirlister.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kio/previewjob.h>
#include <kio/netaccess.h>
#include <qptrlist.h>
#include <kfileitem.h>

// Own includes
#include "kmylistbox.h"
#include "krecursivelister.h"
using namespace KIO;

class FileList : public QPtrList<KFileItem> {
    public:
        FileList() : QPtrList<KFileItem>() {}
        FileList( KFileItemList list ) 
            : QPtrList<KFileItem>() {
                KFileItem* it;
                for( it = list.first(); it; it = list.next() )  
                    this->append( it );
            }

    protected:
        int compareItems( QPtrCollection::Item item1, QPtrCollection::Item item2 ) {
            return static_cast<KFileItem*>(item1)->url().url().compare( static_cast<KFileItem*>(item2)->url().url() );
        }
        
};

KMyListBox::KMyListBox(QWidget* parent, const char* name, WFlags fl)
    :KListBox(parent, name, fl)
{
    drag = ctrlPressed = shiftPressed = mousePressed = false;
    moving = false;
    lister = new KDirLister();
    reclister = new KRecursiveLister();

    setAcceptDrops( true );
    setSelectionMode(Extended); // was extended before 2.9.0

    connect( this, SIGNAL(doubleClicked(QListBoxItem*)), this, SLOT(openFile(QListBoxItem*)));
    connect( this, SIGNAL(returnPressed(QListBoxItem*)), this, SLOT(openFile(QListBoxItem*)));
}

KMyListBox::~KMyListBox()
{
    delete lister;
    delete reclister;
}

KURL KMyListBox::url( int index ) const
{
    return static_cast<KMyListBoxItem*>( item( index ) )->url();
}

bool KMyListBox::dir( int index ) const
{
    return static_cast<KMyListBoxItem*>( item( index ) )->dir();
}

void KMyListBox::setPreviewSize( int size )
{
    previewSize = size;
}

void KMyListBox::addFile( KURL filename, bool isfile )
{
    // Check if the URL is a file and no dir
    // Is the file already in our list ?
    if( !isfile && !isFile( filename ) )
        return;

    if( isInList( filename ) )
        return;

    insertItem( new KMyListBoxItem( filename, false ), -1 );
}

void KMyListBox::addDirName( KURL dirname )
{
   insertItem( new KMyListBoxItem( dirname, true ), -1 );
}

bool KMyListBox::isFile( KURL f )
{
    UDSEntry entry;
    NetAccess::stat( f, entry );
    KFileItem file( entry, f );
    if( !file.isReadable() )
        return false;

    if( file.isDir() ) {
        addDir( f, "*", false, false );
        return false;
    }

    return true;
}

void KMyListBox::addDir( KURL dirname, const QString & m_filter, bool m_hidden, bool recursively, bool dirnames )
{
    if( recursively) {
        reclister->setShowingDotFiles( m_hidden );
        reclister->setNameFilter( m_filter );
        if( dirnames )
            connect( reclister, SIGNAL( completed() ), this, SLOT( listerFinishedRecursivelyDirnames() ) );
        else
            connect( reclister, SIGNAL( completed() ), this, SLOT( listerFinishedRecursivelyNoDirnames() ) );
        reclister->openURL( dirname );
    } else {
        if( dirnames ) {
            QString name = dirname.fileName();
            if( !m_hidden && name.right( 1 ) != QString::fromLatin1(".") )
                if( !isInList( dirname ) )
                    addDirName( dirname );
        }

        lister->setAutoUpdate( false );
        lister->setShowingDotFiles( m_hidden );
        lister->setNameFilter( m_filter );
        connect( lister, SIGNAL( completed() ), this, SLOT( listerFinished() ) );
        lister->openURL( dirname, false, false );
    }
}

void KMyListBox::listerFinished()
{
    if( lister->isFinished() ) {
        FileList list = lister->items( KDirLister::FilteredItems );
        list.sort();
        KFileItem* item;
        for( item = list.first(); item; item = list.next() )
            if( item->isFile() )
                addFile( item->url(), true );

        endAdding();
    }
}

void KMyListBox::listerFinishedRecursivelyDirnames()
{
    listerFinishedRecursively( true );
}

void KMyListBox::listerFinishedRecursivelyNoDirnames()
{
    listerFinishedRecursively( false );
}

void KMyListBox::listerFinishedRecursively( bool dirnames )
{
    KFileItemList l = reclister->items();
    KFileItemList list = l;
    if( dirnames ) {
        FileList dirs = reclister->dirs();
        KFileItem* item;
        for( item = dirs.first(); item; item = dirs.next() )
            list.append( item );
    }

    list.sort();

    KFileItem* item;
    for( item = list.first(); item; item = list.next() )
        if( item->isFile() )
            addFile( item->url(), true );
        else if( item->isDir() )
            addDirName( item->url() );

    endAdding();
}

void KMyListBox::nameListerFinished()
{
    FileList list = reclister->dirs();
    list.sort();
    KFileItem* item;
    for( item = list.first(); item; item = list.next() )
        if( item->isDir() )
            addDirName( item->url() );

    endAdding();
}

void KMyListBox::addDirName( KURL dirname, const QString & m_filter, bool m_hidden, bool recursive )
{
    if( recursive ) {
        addDirName( dirname ); // so that the parent directory is added, too
        reclister = new KRecursiveLister();
        reclister->setShowingDotFiles( m_hidden );
        reclister->setNameFilter( m_filter );
        reclister->setDirOnlyMode( true );
        connect( reclister, SIGNAL( completed() ), this, SLOT( nameListerFinished() ) );
        reclister->openURL( dirname );
    } else {
        // escape hiden directories
        QString name = dirname.fileName();
        if( !m_hidden && name.right( 1 ) != QString::fromLatin1(".") )
            if( !isInList( dirname ) )
                addDirName( dirname );

        endAdding();
    }
}

void KMyListBox::dropEvent(QDropEvent* e)
{
    if( e->source() != this )
        e->accept(QTextDrag::canDecode(e));

    QStringList list;
    if ( QUriDrag::decodeLocalFiles(e, list) ) {
        startAdding();
        for( unsigned int i = 0; i < list.count(); i++ )
            addFile( list[i], false );

        endAdding();
    }
}

void KMyListBox::dragEnterEvent(QDragEnterEvent* e)
{
    if( e->source() != this )
        e->accept(QTextDrag::canDecode(e));
}

void KMyListBox::viewportMousePressEvent( QMouseEvent* e )
{
   if( moving )
        move( index( itemAt( e->pos() ) ) );
    else {
        QListBox::viewportMousePressEvent( e );
        QPoint p( e->pos() );
        QListBoxItem *i = itemAt( p );
        if ( i ) {
                presspos = e->pos();
                mousePressed = TRUE;
        }
    }
}

void KMyListBox::viewportMouseMoveEvent( QMouseEvent* e )
{
    if ( mousePressed && ( presspos - e->pos() ).manhattanLength() > KApplication::startDragDistance() ) {
        mousePressed = FALSE;
        QListBoxItem *item = itemAt( presspos );
        if ( item ) {
            QStringList source = text( index( item ) );
            for( int i = 0; i < (signed int)count(); i++ )
                if( isSelected(i) && (i != index(item) ))
                    source.append( text(i) );
            QUriDrag* ud = new QUriDrag(viewport());
            ud->setUnicodeUris( source );
            ud->drag();
        }
    }
}

void KMyListBox::viewportMouseReleaseEvent( QMouseEvent* e )
{
    mousePressed = FALSE;
    QListBoxItem* item = itemAt( e->pos() );
    if( e->button() != RightButton )
        selected( item );
    QListBox::viewportMouseReleaseEvent( e );
}

void KMyListBox::keyPressEvent( QKeyEvent* e )
{
    /*
     * TODO: Document all this keyboard commands
     */
    if( e->key() == Key_Control )
        ctrlPressed = true;
    else if( e->key() == Key_Shift )
        shiftPressed = true;
    else if( e->key() == Key_Up )
        setCurrentItem( currentItem()-1 );
    else if( e->key() == Key_Down )
        setCurrentItem( currentItem()+1 );
    else if( e->key() == Key_Space )
        select( item(currentItem()) );
    else if( e->key() == Key_Return )
        openFile( item(currentItem()) );
    else if( e->key() == Key_Delete )
        emit deletePressed();
    else if( e->key() == Key_A )
        for( unsigned int i = 0; i < count(); i++ )
            setSelected( i, true );
    else if( e->key() == Key_M ) {
        moveMode();
        return;
    } else if( e->key() == Key_N )
        for( unsigned int i = 0; i < count(); i++ )
            setSelected( i, false );
    else if( e->key() == Key_End )
        setCurrentItem( count()-1 );
    else if( e->key() == Key_Home )
        setCurrentItem( 0 );
    else
        e->ignore();

    emit updateCount();
    setPreview( KMyListBoxItem::preview() );
    emit updatePreview();
}

void KMyListBox::keyReleaseEvent( QKeyEvent* e )
{
    if( e->key() == Key_Control )
        ctrlPressed = false;
    else if( e->key() == Key_Shift )
        shiftPressed = false;
}

void KMyListBox::openFile( QListBoxItem* item )
{
    if( item ) {
        KMyListBoxItem* it = static_cast<KMyListBoxItem*>(item);
        KFileItem* fileItem = new KFileItem( KFileItem::Unknown, KFileItem::Unknown, it->url() );
        fileItem->run();
        delete fileItem;
    }
}

void KMyListBox::moveMode()
{
    if ( !moving ) {
        moving = true;
        KApplication::setOverrideCursor( Qt::sizeAllCursor );
    }
}

void KMyListBox::select( QListBoxItem* item )
{
    if( !ctrlPressed && !shiftPressed )
        /* Single click on the list box,
         * make all items but the clicked
         * one not selected */
        for( int i = 0; i < (signed int)count(); i++ )
            if( i != index(item) )
                setSelected( i, false );

    if( shiftPressed ) {
        if( currentItem() == -1 ) {
            setSelected( item, !isSelected( item ));
            setCurrentItem( item );
            return;
        }
        if( currentItem() > index(item) ) {
            for( int i = index(item); i <= currentItem(); i++ )
                setSelected( i, !isSelected( i ));
            setCurrentItem( item );
        } else if( currentItem() < index(item) ) { /* Works !*/
            for( int i = currentItem()+1; i <= index(item); i++ )
                setSelected( i, !isSelected( i ));
        } else /* c == index(item) */ /* Works !*/
            setSelected( item, !isSelected( item ));
    } else {
        setSelected( item, !isSelected( item ));
        setCurrentItem( item );
    }
}

void KMyListBox::preview( KURL::List list )
{
    KIO::PreviewJob* job = KIO::filePreview( list, previewSize, previewSize, 0, 100, false, true, 0 );
    connect( job, SIGNAL( gotPreview( const KFileItem*, const QPixmap &) ), this, SLOT( previewDone( const KFileItem*, const QPixmap &) ) );
    connect( job, SIGNAL( failed( const KFileItem*)), this, SLOT( previewFailed( const KFileItem* ) ));
    connect( job, SIGNAL( result( KIO::Job * ) ), this, SLOT( previewFinished() ) );
    KApplication::setOverrideCursor( Qt::waitCursor );
}

void KMyListBox::previewDone( const KFileItem* item, const QPixmap &pixmap )
{
    for( unsigned int i = 0; i < count(); i++ )
        if( url( i ) == item->url() ) {
            KMyListBoxItem* it = static_cast<KMyListBoxItem*>(this->item( i ));
            if( it && !pixmap.isNull() ) {
                it->setPixmap( pixmap );
                //updateItem( i );
            }
            break;
        }
}

void KMyListBox::previewFailed( const KFileItem* item )
{
    for( unsigned int i = 0; i < count(); i++ )
        if( url( i ) == item->url() ) {
            KMyListBoxItem* it = static_cast<KMyListBoxItem*>(this->item( i ));
            if( it ) {
                it->setPixmap( item->pixmap( getPreviewSize(), KIcon::DefaultState ) );
            }
            break;
        }
}

void KMyListBox::previewFinished()
{
    triggerUpdate( true ); //maybe false is enough
    KApplication::restoreOverrideCursor();
}

void KMyListBox::setPreview( bool prv )
{
    KMyListBoxItem::setPreview( prv );

    if( prv ) {
        KURL::List list;
        for( unsigned int i = 0; i < count(); i++ ) {
            KMyListBoxItem* it = static_cast<KMyListBoxItem*>(item( i ) );
            if( it->pixmap()->isNull() )
                list.append( it->url() );
        }
        preview( list );
    }
}

void KMyListBox::setName( bool name )
{
    KMyListBoxItem::setName( name );
    setPreview( KMyListBoxItem::preview() );

    if( name ) {
        setColumnMode( FixedNumber );
    } else {
        setColumnMode( FitToWidth );
    }
}

void KMyListBox::move( int i )
{
    KApplication::restoreOverrideCursor();
    moving = false;

    if( !count() )
        return;

    int nbSelectedBefore = 0;
    for( unsigned int j = 0 ; j < count() ; j++ ) {
        if( isSelected( j ) ) {
            if( j < (unsigned int)i )
                nbSelectedBefore++;

            KMyListBoxItem* item = new KMyListBoxItem( static_cast<KMyListBoxItem*>(this->item( j )) );
            removeItem( j );
            j--;
            insertItem( item, i - nbSelectedBefore );
        }
    }
}

void KMyListBox::moveUp()
{
    if( count() == 0 )
        return;

    unsigned int i = 0;
    setUpdatesEnabled( false );
    do {
        if( isSelected( i ) && i ) {
            moveUp( i );

            setSelected( i-1, true );
            setCurrentItem( i-1 );
        }
        i++;
    } while( i < count() );
    setUpdatesEnabled( true );
}

void KMyListBox::moveUp( int i )
{
    if( count() == 0 )
        return;

    KMyListBoxItem* item = new KMyListBoxItem( static_cast<KMyListBoxItem*>(this->item( i )) );
    removeItem( i );
    insertItem( item, i - 1 );
}

void KMyListBox::moveDown()
{
    if( count() == 0 )
        return;

    unsigned int i = count();
    setUpdatesEnabled( false );
    do {
        i--;
        if( isSelected( i ) && (i < count()) ) {
            moveDown( i );
            setSelected( i+1, true );
            setCurrentItem( i+1 );
        }
    } while( i > 0 );
    setUpdatesEnabled( true );
}

void KMyListBox::moveDown( int i )
{
    if( count() == 0 )
        return;

    KMyListBoxItem* item = new KMyListBoxItem( static_cast<KMyListBoxItem*>(this->item( i )) );
    removeItem( i );
    insertItem( item, i + 1 );
}

bool KMyListBox::isInList( KURL text )
{
    // TODO: faster find algorithm
    for( unsigned int i = 0; i < count(); i++ ) {
        KMyListBoxItem* it = static_cast<KMyListBoxItem*>(item( i ) );
        if( it->url() == text ) {
            return true;
        }
    }

    return false;
}

void KMyListBox::startAdding()
{
    setUpdatesEnabled( false );
    KApplication::setOverrideCursor( Qt::waitCursor );
}

void KMyListBox::endAdding()
{
    KApplication::restoreOverrideCursor();
    setUpdatesEnabled( true );
    setPreview( KMyListBoxItem::preview() );

    emit updateCount();
    emit updatePreview();
}

void KMyListBox::sortAscending()
{
    sort( true );
    emit updatePreview();
}

void KMyListBox::sortDescending()
{
    sort( false );
    emit updatePreview();
}

KMyListBoxItem::KMyListBoxItem( const KMyListBoxItem* item )
    : QListBoxItem()
{
    m_url = item->url();
    m_dir = item->dir();
    pm = *item->pixmap();
}

KMyListBoxItem::KMyListBoxItem( const KURL & u, bool b )
    : QListBoxItem()
{
    m_url = u;
    m_dir = b;
    m_preview = m_name = false;
}

void KMyListBoxItem::setPixmap( const QPixmap & pix )
{
    pm = pix;
}

void KMyListBoxItem::setName( bool b )
{
    KMyListBoxItem::m_name = b;
}

void KMyListBoxItem::setPreview( bool b )
{
    KMyListBoxItem::m_preview = b;
}

void KMyListBoxItem::paint( QPainter *painter )
{
    if( !KMyListBoxItem::m_preview ) {
        int itemHeight = height( listBox() );
        QFontMetrics fm = painter->fontMetrics();
        int yPos = ( ( itemHeight - fm.height() ) / 2 ) + fm.ascent();
        painter->drawText( 3, yPos, text() );
    } else {
        int itemHeight = height( listBox() );
        int yPos;

        if( pm.isNull() ) {
            KFileItem item( KFileItem::Unknown, KFileItem::Unknown, m_url );
            KMyListBox* box = static_cast<KMyListBox*>(this->listBox());
            pm = item.pixmap( box->getPreviewSize(), KIcon::DefaultState );
        }

        yPos = ( itemHeight - pm.height() ) / 2;
        painter->drawPixmap( 3, yPos, pm);

        if( KMyListBoxItem::m_name && !m_url.isEmpty() ) {
            QFontMetrics fm = painter->fontMetrics();
            yPos = ( ( itemHeight - fm.height() ) / 2 ) + fm.ascent();
            painter->drawText( pm.width() + 5, yPos, text() );
        }
    }
}

int KMyListBoxItem::height( const QListBox* lb ) const
{
    if( !KMyListBoxItem::m_preview ) {
        int h = listBox() ? listBox()->fontMetrics().lineSpacing() + 2 : 0;
        return QMAX( h, QApplication::globalStrut().height() );
    } else {
        int h;
        if ( KMyListBoxItem::m_name && !m_url.prettyURL().isEmpty() )
            h = pm.height();
        else
            h = QMAX( pm.height(), lb->fontMetrics().lineSpacing() + 2 );

        return QMAX( h, QApplication::globalStrut().height() );
    }
}

int KMyListBoxItem::width( const QListBox* ) const
{
    if( !KMyListBoxItem::m_preview ) {
        int w = listBox() ? listBox()->fontMetrics().width( text() ) + 6 : 0;
        return QMAX( w, QApplication::globalStrut().width() );
    } else {
        if ( m_url.path().isEmpty() || !KMyListBoxItem::m_name)
            return QMAX( pm.width() + 6, QApplication::globalStrut().width() );
        
        return QMAX( pm.width() + listBox()->fontMetrics().width( text() ) + 6, QApplication::globalStrut().width() );
    }
}

QString KMyListBoxItem::text() const
{
    return m_url.prettyURL( ); //0, m_url.isLocalFile() ? KURL::StripFileProtocol : 0 );
}

bool KMyListBoxItem::m_preview = false;
bool KMyListBoxItem::m_name = false;

