/***************************************************************************
                      simplefilenamedlg.cpp  -  description
                             -------------------
    begin                : Fri Apr 09 2004
    copyright            : (C) 2003 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "simplefilenamedlg.h"

#include "kmyhistorycombo.h"
#include "krenameimpl.h"

// Qt includes
#include <qgrid.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qradiobutton.h>
#include <qvbuttongroup.h>
#include <qvgroupbox.h>
#include <qvbox.h>

// KDE includes
#include <kcombobox.h>
#include <klocale.h>
#include <knuminput.h>

SimpleFilenameDlg::SimpleFilenameDlg(KRenameImpl* krename,QWidget *parent, const char *name)
    : KDialogBase( KDialogBase::Plain, i18n("Setup Filename"),
      KDialogBase::Close, KDialogBase::Close, parent, name, true, true ), m_krename( krename )
{
    QVBoxLayout* layout = new QVBoxLayout( plainPage(), 6, 6 );

    QVGroupBox* vgroup1 = new QVGroupBox( i18n("&Filename:"), plainPage() );
    QGrid* grid = new QGrid( 3, vgroup1 );
        
    QLabel* l = new QLabel( i18n("&Prefix:"), grid );
    comboKRenamePrefix = new KComboBox( false, grid );
    comboPrefix = new KMyHistoryCombo( grid, "comboPrefix" );   
    l->setBuddy( comboPrefix );
    
    l = new QLabel(  i18n("&Suffix:"), grid, "comboSuffix" );
    comboKRenameSuffix = new KComboBox( false, grid );
    comboSuffix = new KMyHistoryCombo( grid );
    l->setBuddy( comboSuffix );

    QVButtonGroup* vgroup = new QVButtonGroup( i18n("&Previous Name"), plainPage() );
    radioName = new QRadioButton( i18n("&Use previous name"), vgroup );
    radioLower = new QRadioButton( i18n("Convert to &lower case "), vgroup );
    radioUpper = new QRadioButton( i18n("Convert to &upper case"), vgroup );
    radioFirst = new QRadioButton( i18n("&Capitilazise"), vgroup );
    radioCustom = new QRadioButton( i18n("Custom &name"), vgroup );
    comboCustom = new KMyHistoryCombo( vgroup, "comboCustom" );
                
    QVGroupBox* group = new QVGroupBox( i18n("&Number:" ), plainPage() );
    
    spinStart = new KIntNumInput( group );
    spinStart->setLabel( i18n( "Start &index:" ), AlignLeft | AlignVCenter );
    spinStart->setValue( m_krename->counterStart() );

    spinNull = new KIntNumInput( spinStart, 1, group );        
    spinNull->setLabel( i18n("&Number of digits"), AlignLeft | AlignVCenter );
    spinNull->setRange( 1, 100, 1, false );
    
    layout->addWidget( vgroup1 );
    layout->addWidget( vgroup );
    layout->addWidget( group );
   
    comboKRenamePrefix->insertItem( "" );
    comboKRenamePrefix->insertItem( i18n("Number") );
    comboKRenamePrefix->insertItem( i18n("Date") );
    
    comboKRenameSuffix->insertItem( "" );
    comboKRenameSuffix->insertItem( i18n("Number") );
    comboKRenameSuffix->insertItem( i18n("Date") );
    
    parse();
    enableControls();
        
    connect( comboPrefix, SIGNAL( textChanged( const QString &)), this, SLOT( changed()));
    connect( comboSuffix, SIGNAL( textChanged( const QString &)), this, SLOT( changed()));
    connect( radioName, SIGNAL( clicked() ), this, SLOT( changed() ));
    connect( radioLower, SIGNAL( clicked() ), this, SLOT( changed() ));
    connect( radioUpper, SIGNAL( clicked() ), this, SLOT( changed() ));
    connect( radioFirst, SIGNAL( clicked() ), this, SLOT( changed() ));
    connect( radioCustom, SIGNAL( clicked() ), this, SLOT( changed() ));
    connect( comboCustom, SIGNAL( textChanged( const QString & ) ), this, SLOT( changed() ));
    connect( comboKRenamePrefix, SIGNAL(activated(int)), this, SLOT(changed()));
    connect( comboKRenameSuffix, SIGNAL(activated(int)), this, SLOT(changed()));
    connect( spinNull, SIGNAL( valueChanged(int) ), this, SLOT( changed() ));
    connect( spinStart, SIGNAL( valueChanged(int) ), this, SLOT( changed() ));
}


SimpleFilenameDlg::~SimpleFilenameDlg()
{
    comboPrefix->saveSettings();
    comboSuffix->saveSettings();
    comboCustom->saveSettings();
}

void SimpleFilenameDlg::parse()
{
    /** This function is not very accrurate and 
      * guesses most cases.
      * But it works pretty well and is IMHO enough for this
      * simple dialog.
      */
    QString t = m_krename->fileNameTemplate();
    spinNull->setValue( t.contains( "#" ) );
    
    if( t.startsWith( "[date]") ) 
    {
        t = t.right( t.length() - 6 );
        comboKRenamePrefix->setCurrentItem( 2 );
    }
    else if( t.startsWith( "#") )
    {
        while( t.startsWith( "#" ) )
            t = t.right( t.length() - 1 );
        comboKRenamePrefix->setCurrentItem( 1 );
    }
    
    if( t.endsWith( "[date]" ) )
    {
        t = t.left( t.length() - 6 );
        comboKRenameSuffix->setCurrentItem( 2 );
    }
    else if( t.endsWith( "#") )
    {
        while( t.endsWith( "#" ) )
            t = t.left( t.length() - 1 );
        
        comboKRenameSuffix->setCurrentItem( 1 );
    }
    
    int pos = -1;
    if( (pos = t.find( "$" )) > -1 )
        radioName->setChecked( true );
    else if( (pos = t.find( "%" )) > -1 )
        radioLower->setChecked( true );
    else if( (pos = t.find( "&" )) > -1 )
        radioUpper->setChecked( true );
    else if( (pos = t.find( "*" )) > -1 ) 
        radioFirst->setChecked( true );
    else
    {
        radioCustom->setChecked( true );
        comboCustom->setText( t );
    }
        
    if( pos > 0 )
    {
        comboPrefix->setText( t.left( pos ) );
        comboSuffix->setText( t.mid( pos + 1, t.length() - pos - 1) );
    }
}

void SimpleFilenameDlg::enableControls()
{
    comboCustom->setEnabled( radioCustom->isChecked() );
}

void SimpleFilenameDlg::changed()
{
    enableControls();
    
    QString t = QString::null;
    if( radioName->isChecked() )
        t = "$";
    else if( radioLower->isChecked() )
        t =  "%";
    else if( radioUpper->isChecked() )
        t = "&";
    else if( radioFirst->isChecked() )
        t = "*";
    else if( radioCustom->isChecked() )
        t = comboCustom->text();

    t = comboPrefix->text() + t + comboSuffix->text();

    for( int i=0; i<spinNull->value();i++)
    {
        if( comboKRenamePrefix->currentItem() == 1 )
            t.prepend("#");
        if( comboKRenameSuffix->currentItem() == 1 )
            t.append("#");
    }

    if( comboKRenamePrefix->currentItem() == 2 )        
        t.prepend( "[date]" );
    
    if( comboKRenameSuffix->currentItem() == 2 )
        t.append( "[date]" );

    bool update = false;
    update = ( m_krename->counterStart() != spinStart->value() && t == m_krename->fileNameTemplate() );
    m_krename->setCounterStart( spinStart->value() );
    
    if( update )
        m_krename->updatePre();
                
    emit newTemplate( t );
}

#include "simplefilenamedlg.moc"
